@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,13 +15,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #include "ha_ndbcluster_glue.h"
-
-#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
 #include <ndbapi/NdbApi.hpp>
 #include <portlib/NdbTick.h>
 #include "ha_ndbcluster_connection.h"
@@ -44,14 +38,12 @@ extern int global_flag_skip_waiting_for_clean_cache;
 
 int
 ndbcluster_connect(int (*connect_callback)(void),
-                   ulong wait_connected,
+                   ulong wait_connected, // Timeout in seconds
                    uint connection_pool_size,
                    bool optimized_node_select,
                    const char* connect_string,
                    uint force_nodeid)
 {
-  NDB_TICKS end_time;
-
 #ifndef EMBEDDED_LIBRARY
   const char mysqld_name[]= "mysqld";
 #else
@@ -99,12 +91,12 @@ ndbcluster_connect(int (*connect_callback)(void),
 
   /* Connect to management server */
 
-  end_time= NdbTick_CurrentMillisecond();
-  end_time+= 1000 * wait_connected;
+  const NDB_TICKS start= NdbTick_getCurrentTicks();
 
   while ((res= g_ndb_cluster_connection->connect(0,0,0)) == 1)
   {
-    if (NdbTick_CurrentMillisecond() > end_time)
+    const NDB_TICKS now = NdbTick_getCurrentTicks();
+    if (NdbTick_Elapsed(start,now).seconds() > wait_connected)
       break;
     do_retry_sleep(100);
     if (abort_loop)
@@ -164,12 +156,13 @@ ndbcluster_connect(int (*connect_callback)(void),
                   g_pool[i]->get_connected_host(),
                   g_pool[i]->get_connected_port()));
 
-      NDB_TICKS now_time;
+      Uint64 waited;
       do
       {
         res= g_pool[i]->wait_until_ready(1, 1);
-        now_time= NdbTick_CurrentMillisecond();
-      } while (res != 0 && now_time < end_time);
+        const NDB_TICKS now = NdbTick_getCurrentTicks();
+        waited = NdbTick_Elapsed(start,now).seconds();
+      } while (res != 0 && waited < wait_connected);
 
       const char *msg= 0;
       if (res == 0)
@@ -340,9 +333,12 @@ static ST_FIELD_INFO ndb_transid_mysql_connection_map_fields_info[] =
   { 0, 0, MYSQL_TYPE_NULL, 0, 0, "", SKIP_OPEN_TABLE }
 };
 
+#include <mysql/innodb_priv.h>
+
 static
 int
-ndb_transid_mysql_connection_map_fill_table(THD* thd, TABLE_LIST* tables, COND* cond)
+ndb_transid_mysql_connection_map_fill_table(THD* thd, TABLE_LIST* tables,
+                                            Item*)
 {
   DBUG_ENTER("ndb_transid_mysql_connection_map_fill_table");
 
@@ -416,7 +412,6 @@ struct st_mysql_plugin i_s_ndb_transid_mysql_connection_map_plugin =
   0x0001,
   NULL,
   NULL,
-  NULL
+  NULL,
+  0
 };
-
-#endif /* WITH_NDBCLUSTER_STORAGE_ENGINE */
