@@ -1,4 +1,4 @@
-/* Copyright (C) 2008 MySQL AB
+/* Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +18,9 @@
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 40
+
+
 class EnableComReq  {
   friend class Qmgr;
   friend class Trpman;
@@ -47,4 +50,7 @@ private:
   Uint32 m_nodeIds[NodeBitmask::Size];
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
