@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2004, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,18 +26,15 @@ NdbMutex *g_ndb_connection_mutex = NULL;
 extern class EventLogger * create_event_logger();
 extern void destroy_event_logger(class EventLogger ** g_eventLogger);
 
-// Turn on monotonic timers and conditions by setting
-// this flag before calling ndb_init 
-int g_ndb_init_need_monotonic = 0;
-
 static int ndb_init_called = 0;
 
 extern "C" void NdbMutex_SysInit();
 extern "C" void NdbMutex_SysEnd();
-extern "C" void NdbCondition_initialize(int need_monotonic);
-extern "C" void NdbTick_Init(int need_monotonic);
+extern "C" void NdbCondition_initialize();
 extern "C" int NdbThread_Init();
 extern "C" void NdbThread_End();
+
+extern void NdbTick_Init();
 extern void NdbOut_Init();
 
 extern "C"
@@ -61,12 +58,8 @@ ndb_init_internal()
       exit(1);
     }
   }
-  /* Always turn on monotonic unless on Solaris */  
-#ifndef __sun
-  g_ndb_init_need_monotonic = 1;
-#endif
-  NdbTick_Init(g_ndb_init_need_monotonic);
-  NdbCondition_initialize(g_ndb_init_need_monotonic);
+  NdbTick_Init();
+  NdbCondition_initialize();
   NdbThread_Init();
 }
 
@@ -83,6 +76,14 @@ ndb_init()
       (void)res;
       exit(1);
     }
+    /*
+      Initialize time conversion information
+       - the "time conversion information" structures are primarily
+         used by localtime_r() when converting epoch time into
+         broken-down local time representation.
+    */
+    tzset();
+
     ndb_init_internal();
   }
   return 0;
