@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,8 +32,12 @@
 #include <signaldata/DropTrigImpl.hpp>
 #include <signaldata/TuxMaint.hpp>
 #include <signaldata/AlterIndxImpl.hpp>
+#include <signaldata/ScanFrag.hpp>
 #include "../dblqh/Dblqh.hpp"
 
+#define JAM_FILE_ID 423
+
+
 /* **************************************************************** */
 /* ---------------------------------------------------------------- */
 /* ----------------------- TRIGGER HANDLING ----------------------- */
@@ -340,7 +344,7 @@ Dbtup::createTrigger(Tablerec* table,
     ndbrequire(tmp[i].list != NULL);
 
     TriggerPtr tptr;
-    if (!tmp[i].list->seize(tptr))
+    if (!tmp[i].list->seizeFirst(tptr))
     {
       jam();
       goto err;
@@ -558,6 +562,7 @@ Dbtup::execFIRE_TRIG_REQ(Signal* signal)
   req_struct.TC_index = signal->theData[2];
   req_struct.trans_id1 = signal->theData[3];
   req_struct.trans_id2 = signal->theData[4];
+  req_struct.m_reorg = regOperPtr.p->op_struct.m_reorg;
 
   PagePtr page;
   Tuple_header* tuple_ptr = (Tuple_header*)
@@ -567,6 +572,7 @@ Dbtup::execFIRE_TRIG_REQ(Signal* signal)
   OperationrecPtr lastOperPtr;
   lastOperPtr.i = tuple_ptr->m_operation_ptr_i;
   c_operation_pool.getPtr(lastOperPtr);
+  ndbassert(regOperPtr.p->op_struct.m_reorg == lastOperPtr.p->op_struct.m_reorg);
 
   /**
    * Deferred triggers should fire only once per primary key (per pass)
@@ -1083,24 +1089,6 @@ Dbtup::fireDetachedTriggers(KeyReqStruct *req_struct,
   }
 }
 
-void Dbtup::executeTriggers(KeyReqStruct *req_struct,
-                            DLList<TupTriggerData>& triggerList, 
-                            Operationrec* regOperPtr,
-                            bool disk)
-{
-  TriggerPtr trigPtr;
-  triggerList.first(trigPtr);
-  while (trigPtr.i != RNIL) {
-    jam();
-    executeTrigger(req_struct,
-                   trigPtr.p,
-                   regOperPtr,
-                   disk);
-    triggerList.next(trigPtr);
-
-  }
-}
-
 bool
 Dbtup::check_fire_trigger(const Fragrecord * fragPtrP,
                           const TupTriggerData* trigPtrP,
@@ -1122,7 +1110,7 @@ Dbtup::check_fire_trigger(const Fragrecord * fragPtrP,
     return false;
   case Fragrecord::FS_REORG_COMMIT:
   case Fragrecord::FS_REORG_COMPLETE:
-    return req_struct->m_reorg == 0;
+    return req_struct->m_reorg == ScanFragReq::REORG_ALL;
   default:
     return true;
   }
@@ -1138,7 +1126,7 @@ Dbtup::check_fire_reorg(const KeyReqStruct *req_struct,
   case Fragrecord::FS_REORG_COMMIT_NEW:
   case Fragrecord::FS_REORG_COMPLETE_NEW:
     jam();
-    if (flag == 2)
+    if (flag == ScanFragReq::REORG_MOVED)
     {
       jam();
       return true;
@@ -1185,7 +1173,7 @@ Dbtup::check_fire_suma(const KeyReqStruct *req_struct,
     return true;
   case Fragrecord::FS_REORG_COMPLETE:
     jam();
-    if (flag != 1)
+    if (flag != ScanFragReq::REORG_NOT_MOVED)
     {
       jam();
       return true;
@@ -1637,6 +1625,27 @@ bool Dbtup::readTriggerInfo(TupTriggerData* const trigPtr,
   req_struct->check_offset[DD]= regTabPtr->get_check_offset(DD);
   req_struct->attr_descr= &tableDescriptor[descr_start];
 
+  if ((regOperPtr->op_struct.m_physical_only_op == 1) &&
+      (refToMain(trigPtr->m_receiverRef) == SUMA ||
+       refToMain(trigPtr->m_receiverRef) == BACKUP))
+  {
+    /* Operations that have no logical effect need not be backed up
+     * or sent as an event. Eg. OPTIMIZE TABLE is performed as a
+     * ZUPDATE operation on table records, moving the varpart 
+     * column-values between pages, to be storage-effective.
+     */
+    Uint32 changed_attribs = 0;
+    for (Uint32 i = 0; i < regTabPtr->m_no_of_attributes; i++) {
+      jam();
+      if (req_struct->changeMask.get(i)) {
+        jam();
+        changed_attribs++;
+      }
+    }
+    ndbrequire(changed_attribs == 0);
+    return false;
+  }
+
 //--------------------------------------------------------------------
 // Read Primary Key Values
 //--------------------------------------------------------------------
