@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,22 +19,27 @@
 #define ThreadConfig_H
 
 #include <kernel_types.h>
-#include <ErrorReporter.hpp>
 #include <NodeState.hpp>
+#include <portlib/NdbThread.h>
+
+#define JAM_FILE_ID 231
 
-class IPCConfig;
 
 class ThreadConfig
 {
 public:
   ThreadConfig();
   ~ThreadConfig();
+  void init(void);
 
-  void ipControlLoop(Uint32 thread_index);
+  void ipControlLoop(NdbThread*);
 
   int doStart(NodeState::StartLevel startLevel);
 private:
 
   void scanTimeQueue();
 };
+
+#undef JAM_FILE_ID
+
 #endif // ThreadConfig_H
