@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,6 +41,28 @@ extern "C" {
 					   int value,
 					   struct ndb_mgm_reply* reply);
 
+  /**
+   * Send list of dynamic ports to use when setting up connections
+   * between nodes in the cluster.
+   *
+   * NOTE! Currently only ndbd's set up dynamic listening ports
+   * and all other node types are clients or have static server ports.
+   *
+   * @param handle the NDB management handle.
+   * @param nodeid the node which has openened the ports
+   * @param ports pointer to an array of ndb_mgm_dynamic_port structs
+   * @param num_ports the number of ndb_mgm_dynamic_ports passed
+   * @return 0 on success. < 0 on error.
+   */
+  struct ndb_mgm_dynamic_port {
+   int nodeid; /* The node which should use below port */
+   int port; /* The port to use */
+  };
+  int ndb_mgm_set_dynamic_ports(NdbMgmHandle handle,
+                                int nodeid,
+                                struct ndb_mgm_dynamic_port* ports,
+                                unsigned num_ports);
+
   /**
    * Get an integer parameter for a connection
    *
