@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2008 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,19 +15,8 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#include <ndb_global.h>
-#include <NdbTransaction.hpp>
-#include <NdbOperation.hpp>
-#include "NdbApiSignal.hpp"
-#include "NdbRecAttr.hpp"
-#include "NdbUtil.hpp"
-#include "NdbBlob.hpp"
-#include "ndbapi_limits.h"
-#include <signaldata/TcKeyReq.hpp>
-#include "NdbDictionaryImpl.hpp"
-
 #include "API.hpp"
-#include <NdbOut.hpp>
+#include <signaldata/TcKeyReq.hpp>
 
 
 /******************************************************************************
@@ -80,9 +68,11 @@ NdbOperation::NdbOperation(Ndb* aNdb, NdbOperation::Type aType) :
   m_attrInfoGSN(GSN_ATTRINFO),
   theBlobList(NULL),
   m_abortOption(-1),
-  m_noErrorPropagation(false)
+  m_noErrorPropagation(false),
+  theLockHandle(NULL),
+  m_blob_lock_upgraded(false)
 {
-  theReceiver.init(NdbReceiver::NDB_OPERATION, false, this);
+  theReceiver.init(NdbReceiver::NDB_OPERATION, this);
   theError.code = 0;
   m_customData = NULL;
 }
@@ -143,8 +133,8 @@ NdbOperation::setErrorCodeAbort(int anErrorCode) const
  *****************************************************************************/
 
 int
-NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection,
-                   bool useRec){
+NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection)
+{
   NdbApiSignal* tSignal;
   theStatus		= Init;
   theError.code		= 0;
@@ -179,11 +169,11 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection,
   theBlobList = NULL;
   m_abortOption = -1;
   m_noErrorPropagation = false;
-  m_no_disk_flag = 1;
+  m_flags = 0;
+  m_flags |= OF_NO_DISK;
   m_interpreted_code = NULL;
   m_extraSetValues = NULL;
   m_numExtraSetValues = 0;
-  m_use_any_value = 0;
 
   tSignal = theNdb->getSignal();
   if (tSignal == NULL)
@@ -192,19 +182,23 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection,
     return -1;
   }
   theTCREQ = tSignal;
-  theTCREQ->setSignal(m_tcReqGSN);
+  theTCREQ->setSignal(m_tcReqGSN, refToBlock(theNdbCon->m_tcRef));
 
   theAI_LenInCurrAI = 20;
   TcKeyReq * const tcKeyReq = CAST_PTR(TcKeyReq, theTCREQ->getDataPtrSend());
   tcKeyReq->scanInfo = 0;
   theKEYINFOptr = &tcKeyReq->keyInfo[0];
   theATTRINFOptr = &tcKeyReq->attrInfo[0];
-  if (theReceiver.init(NdbReceiver::NDB_OPERATION, useRec, this))
+  if (theReceiver.init(NdbReceiver::NDB_OPERATION, this))
   {
     // theReceiver sets the error code of its owner
     return -1;
   }
   m_customData = NULL;
+
+  if (theNdb->theImpl->get_ndbapi_config_parameters().m_default_queue_option)
+    m_flags |= OF_QUEUEABLE;
+
   return 0;
 }
 
@@ -216,6 +210,29 @@ NdbOperation::init(const NdbTableImpl* tab, NdbTransaction* myConnection,
  *****************************************************************************/
 void
 NdbOperation::release()
+{
+  NdbBlob* tBlob;
+  NdbBlob* tSaveBlob;
+
+  /* In case we didn't execute... */
+  postExecuteRelease();
+
+  tBlob = theBlobList;
+  while (tBlob != NULL)
+  {
+    tSaveBlob = tBlob;
+    tBlob = tBlob->theNext;
+    theNdb->releaseNdbBlob(tSaveBlob);
+  }
+  theBlobList = NULL;
+  theReceiver.release();
+
+  theLockHandle = NULL;
+  m_blob_lock_upgraded = false;
+}
+
+void
+NdbOperation::postExecuteRelease()
 {
   NdbApiSignal* tSignal;
   NdbApiSignal* tSaveSignal;
@@ -227,19 +244,25 @@ NdbOperation::release()
   NdbCall*	tSaveCall;
   NdbSubroutine* tSubroutine;
   NdbSubroutine* tSaveSubroutine;
-  NdbBlob* tBlob;
-  NdbBlob* tSaveBlob;
 
-  tSignal = theTCREQ;
+  tSignal = theRequest; /* TCKEYREQ/TCINDXREQ/SCANTABREQ */
   while (tSignal != NULL)
   {
     tSaveSignal = tSignal;
     tSignal = tSignal->next();
     theNdb->releaseSignal(tSaveSignal);
   }				
-  theTCREQ = NULL;
+  theRequest = NULL;
   theLastKEYINFO = NULL;
-
+#ifdef TODO
+  /**
+   * Compute correct #cnt signals between theFirstATTRINFO/theCurrentATTRINFO
+   */
+  if (theFirstATTRINFO)
+  {
+    theNdb->releaseSignals(1, theFirstATTRINFO, theCurrentATTRINFO);
+  }
+#else
   tSignal = theFirstATTRINFO;
   while (tSignal != NULL)
   {
@@ -247,6 +270,7 @@ NdbOperation::release()
     tSignal = tSignal->next();
     theNdb->releaseSignal(tSaveSignal);
   }
+#endif
   theFirstATTRINFO = NULL;
   theCurrentATTRINFO = NULL;
 
@@ -281,15 +305,6 @@ NdbOperation::release()
       theNdb->releaseNdbSubroutine(tSaveSubroutine);
     }
   }
-  tBlob = theBlobList;
-  while (tBlob != NULL)
-  {
-    tSaveBlob = tBlob;
-    tBlob = tBlob->theNext;
-    theNdb->releaseNdbBlob(tSaveBlob);
-  }
-  theBlobList = NULL;
-  theReceiver.release();
 }
 
 NdbRecAttr*
@@ -532,3 +547,80 @@ NdbOperation::getNdbTransaction() const
 {
   return theNdbCon; 
 }
+
+int
+NdbOperation::getLockHandleImpl()
+{
+  assert(! theLockHandle);
+  
+  if (unlikely(theNdb->getMinDbNodeVersion() < 
+               NDBD_UNLOCK_OP_SUPPORTED))
+  {
+    /* Function not implemented yet */
+    return 4003;
+  }
+
+  if (likely(((theOperationType == ReadRequest) ||
+              (theOperationType == ReadExclusive)) &&
+             (m_type == PrimaryKeyAccess) &&
+             ((theLockMode == LM_Read) |
+              (theLockMode == LM_Exclusive))))
+  {
+    theLockHandle = theNdbCon->getLockHandle();
+    if (!theLockHandle)
+    {
+      return 4000;
+    }
+    
+    /* Now operation has a LockHandle - it'll be
+     * filled-in when the operation is prepared prior
+     * to execution.
+     */
+    assert(theLockHandle->m_state == NdbLockHandle::ALLOCATED);
+    assert(! theLockHandle->isLockRefValid());
+    
+    return 0;
+  }
+  else
+  {
+    /* getLockHandle only supported for primary key read with a lock */
+    return 4549;
+  }  
+}
+
+const NdbLockHandle*
+NdbOperation::getLockHandle()
+{
+  if (likely (! m_blob_lock_upgraded))
+  {
+    if (theLockHandle == NULL)
+    {
+      int rc = getLockHandleImpl();
+      
+      if (likely(rc == 0))
+        return theLockHandle;
+      else
+      {
+        setErrorCode(rc);
+        return NULL;
+      }
+    }
+    /* Return existing LockHandle */
+    return theLockHandle;
+  }
+  else
+  {
+    /* Not allowed to call getLockHandle() on a Blob-upgraded
+     * read
+     */
+    setErrorCode(4549);
+    return NULL;
+  } 
+}
+  
+const NdbLockHandle*
+NdbOperation::getLockHandle() const
+{
+  /* NdbRecord / handle already exists variant */
+  return theLockHandle;
+}
