@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -321,11 +321,25 @@ RestoreMetaData::readMetaTableList() {
   const Uint32 tabCount = sectionInfo[1] - 2;
 
   void *tmp;
-  if (buffer_get_ptr(&tmp, 4, tabCount) != tabCount){
-    err << "readMetaTableList read tabCount error" << endl;
+  Uint32 tabsRead = 0;
+  while (tabsRead < tabCount){
+    int count = buffer_get_ptr(&tmp, 4, tabCount - tabsRead);
+    if(count == 0)
+      break;
+    tabsRead += count;
+  }
+  if (tabsRead != tabCount){
+    err << "readMetaTableList read tabCount error, expected count = " << tabCount << ", actual count =" << tabsRead << endl;
     return 0;
   }
-  
+#ifdef ERROR_INSERT
+  if(m_error_insert == NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER)
+  {
+    // clear error insert
+    m_error_insert = 0;
+    m_buffer_sz = BUFFER_SIZE;
+  }
+#endif  
   return tabCount;
 }
 
@@ -724,6 +738,10 @@ TableS::TableS(Uint32 version, NdbTableImpl* tableImpl)
   
   for (int i = 0; i < tableImpl->getNoOfColumns(); i++)
     createAttr(tableImpl->getColumn(i));
+
+  m_staging = false;
+  m_stagingTable = NULL;
+  m_stagingFlags = 0;
 }
 
 TableS::~TableS()
@@ -785,6 +803,19 @@ RestoreDataIterator::RestoreDataIterator(const RestoreMetaData & md, void (* _fr
   m_row_bitfield_len = 0;
 }
 
+
+bool
+RestoreDataIterator::validateRestoreDataIterator()
+{
+    if (!m_bitfield_storage_ptr)
+    {
+        err << "m_bitfield_storage_ptr is NULL" << endl;
+        return false;
+    }
+    return true;
+}
+
+
 RestoreDataIterator::~RestoreDataIterator()
 {
   free_bitfield_storage();
@@ -1304,7 +1335,7 @@ BackupFile::BackupFile(void (* _free_data_callback)())
   m_path[0] = 0;
   m_fileName[0] = 0;
 
-  m_buffer_sz = 64*1024;
+  m_buffer_sz = BUFFER_SIZE;
   m_buffer = malloc(m_buffer_sz);
   m_buffer_ptr = m_buffer;
   m_buffer_data_left = 0;
@@ -1312,6 +1343,20 @@ BackupFile::BackupFile(void (* _free_data_callback)())
   m_file_size = 0;
   m_file_pos = 0;
   m_is_undolog = false;
+#ifdef ERROR_INSERT
+  m_error_insert = 0;
+#endif
+}
+
+bool
+BackupFile::validateBackupFile()
+{
+    if (!m_buffer)
+    {
+        err << "m_buffer is NULL" << endl;
+        return false;
+    }
+    return true;
 }
 
 BackupFile::~BackupFile()
@@ -1655,6 +1700,20 @@ BackupFile::validateFooter(){
   return true;
 }
 
+#ifdef ERROR_INSERT
+void BackupFile::error_insert(unsigned int code)
+{
+  if(code == NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER)
+  {
+    // Reduce size of buffer to test buffer overflow
+    // handling. The buffer must still be large enough to
+    // accommodate the file header.
+    m_buffer_sz = 256;
+    m_error_insert = NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER;
+  }
+}
+#endif
+
 bool RestoreDataIterator::readFragmentHeader(int & ret, Uint32 *fragmentId)
 {
   BackupFormat::DataFile::FragmentHeader Header;
@@ -1757,6 +1816,8 @@ AttributeDesc::AttributeDesc(NdbDictionary::Column *c)
 {
   size = 8*NdbColumnImpl::getImpl(* c).m_attrSize;
   arraySize = NdbColumnImpl::getImpl(* c).m_arraySize;
+  staging = false;
+  parameterSz = 0;
 }
 
 void TableS::createAttr(NdbDictionary::Column *column)
