@@ -1,5 +1,5 @@
-/* Copyright (c) 2003-2007 MySQL AB
-
+/*
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 // Restore
 
@@ -103,6 +104,7 @@ struct AttributeDesc {
   Uint32 m_nullBitIndex;
   AttrConvertFunc convertFunc;
   void *parameter;
+  Uint32 parameterSz; 
   bool truncation_detected;
   bool staging;
 
@@ -368,6 +370,7 @@ protected:
 public:
   bool readHeader();
   bool validateFooter();
+  bool validateBackupFile();
 
   const char * getPath() const { return m_path;}
   const char * getFilename() const { return m_fileName;}
@@ -381,7 +384,7 @@ public:
 #ifdef ERROR_INSERT
   void error_insert(unsigned int code); 
 #endif
-
+  static const Uint32 BUFFER_SIZE = 128*1024;
 private:
   void
   twiddle_atribute(const AttributeDesc * const attr_desc,
@@ -449,6 +452,7 @@ public:
   // Read data file fragment header
   bool readFragmentHeader(int & res, Uint32 *fragmentId);
   bool validateFragmentFooter();
+  bool validateRestoreDataIterator();
 
   const TupleS *getNextTuple(int & res);
   TableS *getCurrentTable();
