@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +30,9 @@
 #include "consumer_printer.hpp"
 #include "../src/ndbapi/NdbDictionaryImpl.hpp"
 
+#define TMP_TABLE_PREFIX "#sql"
+#define TMP_TABLE_PREFIX_LEN 4
+
 extern FilteredNdbOut err;
 extern FilteredNdbOut info;
 extern FilteredNdbOut debug;
@@ -58,7 +61,7 @@ const char *opt_ndb_table= NULL;
 unsigned int opt_verbose;
 unsigned int opt_hex_format;
 unsigned int opt_progress_frequency;
-NDB_TICKS g_report_next;
+NDB_TICKS g_report_prev;
 Vector<BaseString> g_databases;
 Vector<BaseString> g_tables;
 Vector<BaseString> g_include_tables, g_exclude_tables;
@@ -88,6 +91,11 @@ static bool ga_restore = false;
 static bool ga_print = false;
 static bool ga_skip_table_check = false;
 static bool ga_exclude_missing_columns = false;
+static bool ga_exclude_missing_tables = false;
+static bool opt_exclude_intermediate_sql_tables = true;
+#ifdef ERROR_INSERT 
+static unsigned int _error_insert = 0;
+#endif
 static int _print = 0;
 static int _print_meta = 0;
 static int _print_data = 0;
@@ -124,6 +132,7 @@ static const char *opt_include_tables= NULL;
 static const char *opt_exclude_databases= NULL;
 static const char *opt_include_databases= NULL;
 static const char *opt_rewrite_database= NULL;
+static bool opt_restore_privilege_tables = false;
 
 static struct my_option my_long_options[] =
 {
@@ -266,11 +275,26 @@ static struct my_option my_long_options[] =
     "Example: db1.t1,db3.t1",
     (uchar**) &opt_exclude_tables, (uchar**) &opt_exclude_tables, 0,
     GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+  { "restore-privilege-tables", NDB_OPT_NOSHORT,
+    "Restore privilege tables (after they have been moved to ndb)",
+    (uchar**) &opt_restore_privilege_tables,
+    (uchar**) &opt_restore_privilege_tables, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { "exclude-missing-columns", NDB_OPT_NOSHORT,
     "Ignore columns present in backup but not in database",
     (uchar**) &ga_exclude_missing_columns,
     (uchar**) &ga_exclude_missing_columns, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "exclude-missing-tables", NDB_OPT_NOSHORT,
+    "Ignore tables present in backup but not in database",
+    (uchar**) &ga_exclude_missing_tables,
+    (uchar**) &ga_exclude_missing_tables, 0,
+    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+  { "exclude-intermediate-sql-tables", NDB_OPT_NOSHORT,
+    "Do not restore intermediate tables with #sql-prefixed names",
+    (uchar**) &opt_exclude_intermediate_sql_tables,
+    (uchar**) &opt_exclude_intermediate_sql_tables, 0,
+    GET_BOOL, NO_ARG, 1, 0, 0, 0, 0, 0 },
   { "disable-indexes", NDB_OPT_NOSHORT,
     "Disable indexes",
     (uchar**) &ga_disable_indexes,
@@ -287,6 +311,12 @@ static struct my_option my_long_options[] =
   { "skip-broken-objects", 256, "Skip broken object when parsing backup",
     (uchar**) &ga_skip_broken_objects, (uchar**) &ga_skip_broken_objects, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
+#ifdef ERROR_INSERT
+  { "error-insert", NDB_OPT_NOSHORT,
+    "Insert errors (testing option)",
+    (uchar **)&_error_insert, (uchar **)&_error_insert, 0,
+    GET_INT, REQUIRED_ARG, 0, 0, 0, 0, 0, 0 },
+#endif
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -495,18 +525,19 @@ processTableList(const char* str, Vector<BaseString> &lst)
 {
   // Process tables list like db1.t1,db2.t1 and exits when
   // it finds problems.
+  Vector<BaseString> tmp;
   unsigned int i;
   /* Split passed string on comma into 2 BaseStrings in the vector */
-  BaseString(str).split(lst,",");
-  for (i=0; i < lst.size(); i++)
+  BaseString(str).split(tmp,",");
+  for (i=0; i < tmp.size(); i++)
   {
     BaseString internalName;
-    if (makeInternalTableName(lst[i], internalName))
+    if (makeInternalTableName(tmp[i], internalName))
     {
-      info << "`" << lst[i] << "` is not a valid tablename!" << endl;
+      info << "`" << tmp[i] << "` is not a valid tablename!" << endl;
       exit(NDBT_ProgramExit(NDBT_WRONGARGS));
     }
-    lst[i].assign(internalName);
+    lst.push_back(internalName);
   }
 }
 
@@ -524,6 +555,24 @@ makeExternalTableName(const BaseString &internalName)
   return externalName;
 }
 
+#include "../../../../sql/ndb_dist_priv_util.h"
+
+// Exclude privilege tables unless explicitely included
+void
+exclude_privilege_tables()
+{
+  const char* table_name;
+  Ndb_dist_priv_util dist_priv;
+  while((table_name= dist_priv.iter_next_table()))
+  {
+    BaseString priv_tab;
+    priv_tab.assfmt("%s.%s", dist_priv.database(), table_name);
+    g_exclude_tables.push_back(priv_tab);
+    save_include_exclude(OPT_EXCLUDE_TABLES, (char *)priv_tab.c_str());
+  }
+}
+
+
 bool
 readArguments(int *pargc, char*** pargv) 
 {
@@ -533,7 +582,7 @@ readArguments(int *pargc, char*** pargv)
   const char *load_default_groups[]= { "mysql_cluster","ndb_restore",0 };
 
   init_nodegroup_map();
-  load_defaults("my",load_default_groups,pargc,pargv);
+  ndb_load_defaults(NULL,load_default_groups,pargc,pargv);
   debug << "handle_options" << endl;
 
   ndb_opt_set_usage_funcs(short_usage_sub, usage);
@@ -581,6 +630,7 @@ o verify nodegroup mapping
                                              opt_ndb_nodeid,
                                              opt_nodegroup_map,
                                              opt_nodegroup_map_len,
+                                             ga_nodeId,
                                              ga_nParallelism);
   if (restore == NULL) 
   {
@@ -621,6 +671,14 @@ o verify nodegroup mapping
   {
     //    ga_restore = true;
     restore->m_restore_meta = true;
+    if(ga_exclude_missing_tables)
+    {
+      //conflict in options
+      err << "Conflicting arguments found : "
+          << "Cannot use `restore-meta` and "
+          << "`exclude-missing-tables` together. Exiting..." << endl;
+      return false;
+    }
   }
 
   if (_no_restore_disk)
@@ -698,7 +756,35 @@ o verify nodegroup mapping
     if (g_tables.size() > 0)
       info << endl;
   }
-  
+
+  if (ga_restore)
+  {
+    // Exclude privilege tables unless explicitely included
+    if (!opt_restore_privilege_tables)
+      exclude_privilege_tables();
+
+    // Move over old style arguments to include/exclude lists
+    if (g_databases.size() > 0)
+    {
+      BaseString tab_prefix, tab;
+      tab_prefix.append(g_databases[0].c_str());
+      tab_prefix.append(".");
+      if (g_tables.size() == 0)
+      {
+        g_include_databases.push_back(g_databases[0]);
+        save_include_exclude(OPT_INCLUDE_DATABASES,
+                             (char *)g_databases[0].c_str());
+      }
+      for (unsigned i= 0; i < g_tables.size(); i++)
+      {
+        tab.assign(tab_prefix);
+        tab.append(g_tables[i]);
+        g_include_tables.push_back(tab);
+        save_include_exclude(OPT_INCLUDE_TABLES, (char *)tab.c_str());
+      }
+    }
+  }
+
   if (opt_include_databases)
   {
     tmp = BaseString(opt_include_databases);
@@ -758,7 +844,7 @@ o verify nodegroup mapping
     }
     info << endl;
   }
-  
+
   /*
     the below formatting follows the formatting from mysqldump
     do not change unless to adopt to changes in mysqldump
@@ -978,10 +1064,21 @@ static bool check_include_exclude(BaseString database, BaseString table)
       }
     }
   }
-  
+
   return do_include;
 }
 
+static bool
+check_intermediate_sql_table(const char *table_name)
+{
+  BaseString tbl(table_name);
+  Vector<BaseString> fields;
+  tbl.split(fields, "/");
+  if((fields.size() == 3) && !fields[2].empty() && strncmp(fields[2].c_str(), TMP_TABLE_PREFIX, TMP_TABLE_PREFIX_LEN) == 0) 
+    return true;  
+  return false;
+}
+  
 static inline bool
 checkDoRestore(const TableS* table)
 {
@@ -1011,8 +1108,13 @@ checkDbAndTableName(const TableS* table)
   if (table->isBroken())
     return false;
 
+  const char *table_name = getTableName(table);
+  if(opt_exclude_intermediate_sql_tables && (check_intermediate_sql_table(table_name) == true)) {
+    return false;
+  }
+
   // If new options are given, ignore the old format
-  if (opt_include_tables || opt_exclude_tables ||
+  if (opt_include_tables || g_exclude_tables.size() > 0 ||
       opt_include_databases || opt_exclude_databases ) {
     return (checkDoRestore(table));
   }
@@ -1024,8 +1126,6 @@ checkDbAndTableName(const TableS* table)
     g_databases.push_back("TEST_DB");
 
   // Filter on the main table name for indexes and blobs
-  const char *table_name= getTableName(table);
-
   unsigned i;
   for (i= 0; i < g_databases.size(); i++)
   {
@@ -1056,6 +1156,38 @@ checkDbAndTableName(const TableS* table)
   return false;
 }
 
+static void
+exclude_missing_tables(const RestoreMetaData& metaData)
+{
+  Uint32 i, j;
+  bool isMissing;
+  Vector<BaseString> missingTables;
+  for(i = 0; i < metaData.getNoOfTables(); i++)
+  {
+    const TableS *table= metaData[i];
+    isMissing = false;
+    for(j = 0; j < g_consumers.size(); j++)
+      isMissing |= g_consumers[j]->isMissingTable(*table);
+    if( isMissing )
+    {
+      /* add missing tables to exclude list */
+      g_exclude_tables.push_back(table->getTableName());
+      BaseString tableName = makeExternalTableName(table->getTableName());
+      save_include_exclude(OPT_EXCLUDE_TABLES, (char*)tableName.c_str());
+      missingTables.push_back(tableName);
+    }
+  }
+
+  if(missingTables.size() > 0){
+    info << "Excluded Missing tables: ";
+    for (i=0 ; i < missingTables.size(); i++)
+    {
+      info << missingTables[i] << " ";
+    }
+    info << endl;
+  }
+}
+
 static void
 free_data_callback()
 {
@@ -1074,8 +1206,7 @@ static void exitHandler(int code)
 
 static void init_progress()
 {
-  Uint64 now = NdbTick_CurrentMillisecond() / 1000;
-  g_report_next = now + opt_progress_frequency;
+  g_report_prev = NdbTick_getCurrentTicks();
 }
 
 static int check_progress()
@@ -1083,11 +1214,11 @@ static int check_progress()
   if (!opt_progress_frequency)
     return 0;
 
-  NDB_TICKS now = NdbTick_CurrentMillisecond() / 1000;
+  const NDB_TICKS now = NdbTick_getCurrentTicks();
   
-  if (now  >= g_report_next)
+  if (NdbTick_Elapsed(g_report_prev, now).seconds() >= opt_progress_frequency)
   {
-    g_report_next = now + opt_progress_frequency;
+    g_report_prev = now;
     return 1;
   }
   return 0;
@@ -1157,6 +1288,8 @@ main(int argc, char** argv)
     g_options.appfmt(" -d");
   if (ga_exclude_missing_columns)
     g_options.append(" --exclude-missing-columns");
+  if (ga_exclude_missing_tables)
+    g_options.append(" --exclude-missing-tables");
   if (ga_disable_indexes)
     g_options.append(" --disable-indexes");
   if (ga_rebuild_indexes)
@@ -1174,6 +1307,13 @@ main(int argc, char** argv)
    */
   debug << "Start restoring meta data" << endl;
   RestoreMetaData metaData(ga_backupPath, ga_nodeId, ga_backupId);
+#ifdef ERROR_INSERTED 
+  if(_error_insert > 0)
+  {
+    metaData.error_insert(_error_insert);
+  }
+#endif 
+
   if (!metaData.readHeader())
   {
     err << "Failed to read " << metaData.getFilename() << endl << endl;
@@ -1256,6 +1396,9 @@ main(int argc, char** argv)
 
   }
 
+  if(ga_exclude_missing_tables)
+    exclude_missing_tables(metaData);
+
   /* report to clusterlog if applicable */
   for (i = 0; i < g_consumers.size(); i++)
     g_consumers[i]->report_started(ga_backupId, ga_nodeId);
@@ -1367,18 +1510,61 @@ main(int argc, char** argv)
         if (checkSysTable(metaData, i) &&
             checkDbAndTableName(metaData[i]))
         {
+          TableS & tableS = *metaData[i]; // not const
           for(Uint32 j= 0; j < g_consumers.size(); j++)
           {
-            if (!g_consumers[j]->table_compatible_check(*metaData[i]))
+            if (!g_consumers[j]->table_compatible_check(tableS))
             {
               err << "Restore: Failed to restore data, ";
-              err << metaData[i]->getTableName() << " table structure incompatible with backup's ... Exiting " << endl;
+              err << tableS.getTableName() << " table structure incompatible with backup's ... Exiting " << endl;
               exitHandler(NDBT_FAILED);
             } 
+            if (tableS.m_staging &&
+                !g_consumers[j]->prepare_staging(tableS))
+            {
+              err << "Restore: Failed to restore data, ";
+              err << tableS.getTableName() << " failed to prepare staging table for data conversion ... Exiting " << endl;
+              exitHandler(NDBT_FAILED);
+            }
           } 
         }
       }
+      for (i=0; i < metaData.getNoOfTables(); i++)
+      {
+        if (checkSysTable(metaData, i) &&
+            checkDbAndTableName(metaData[i]))
+        {
+          // blob table checks use data which is populated by table compatibility checks
+          TableS & tableS = *metaData[i];
+          if(isBlobTable(&tableS))
+          {
+            for(Uint32 j= 0; j < g_consumers.size(); j++)
+            {
+              if (!g_consumers[j]->check_blobs(tableS))
+              {
+                  err << "Restore: Failed to restore data, ";
+                  err << tableS.getTableName() << " table's blobs incompatible with backup's ... Exiting " << endl;
+                  exitHandler(NDBT_FAILED);
+              }
+            }
+          }
+        }
+      }
+        
       RestoreDataIterator dataIter(metaData, &free_data_callback);
+
+      if (!dataIter.validateBackupFile())
+      {
+          err << "Unable to allocate memory for BackupFile constructor" << endl;
+          exitHandler(NDBT_FAILED);
+      }
+
+
+      if (!dataIter.validateRestoreDataIterator())
+      {
+          err << "Unable to allocate memory for RestoreDataIterator constructor" << endl;
+          exitHandler(NDBT_FAILED);
+      }
       
       // Read data file header
       if (!dataIter.readHeader())
@@ -1478,6 +1664,28 @@ main(int argc, char** argv)
       }
     }
     
+    /* move data from staging table to real table */
+    if(_restore_data)
+    {
+      for (i = 0; i < metaData.getNoOfTables(); i++)
+      {
+        const TableS* table = metaData[i];
+        if (table->m_staging)
+        {
+          for(Uint32 j= 0; j < g_consumers.size(); j++)
+          {
+            if (!g_consumers[j]->finalize_staging(*table))
+            {
+              err << "Restore: Failed staging data to table: ";
+              err << table->getTableName() << ". ";
+              err << "Exiting... " << endl;
+              exitHandler(NDBT_FAILED);
+            }
+          }
+        }
+      }
+    }
+
     if(_restore_data)
     {
       for(i = 0; i < metaData.getNoOfTables(); i++)
